<?php
///
// Isolde (Revision r168core-r128app)
//
// A third party package for CodeIgniter
//
// Copyright (c) 2013 Kenta Saito (gatedruid.com)
// License       MIT License (See ../isolde_license.txt)
//
?>
<?php
if ( ! function_exists('template'))
{
	function template($template, $vars = array(), $ignore = FALSE, $return = FALSE)
	{

		$CI =& get_instance();

		$isolde = (object) array();

		$isolde->modules = $CI->isolde->modules;
		$isolde->segments = $CI->isolde->segments;
		$isolde->auth = $CI->isolde->auth;
		$isolde->crud = $CI->isolde->crud;
		$isolde->config = $CI->isolde->config;

		$flash_messages = $CI->session->flashdata('messages');

		$vars = (object) $vars;

		$template_candidates = array(
			"{$CI->isolde->segments->actor}/{$template}",
			"{$template}",
			"../third_party/isolde_core/views/{$template}",
			"_isolde/{$template}",
			"../third_party/isolde_core/views/_isolde/{$template}",
		);
		if (in_array('auth', $CI->isolde->modules))
		{
			$template_candidates = array_merge(
				$template_candidates,
				array(
					"_auth/{$template}",
					"../third_party/isolde_core/views/_auth/{$template}",
				)
			);
		}
		if (in_array('crud', $CI->isolde->modules))
		{
			$template_candidates = array_merge(
				$template_candidates,
				array(
					"{$CI->isolde->segments->actor}/{$CI->isolde->segments->target}/{$template}",
					"{$CI->isolde->segments->target}/{$template}",
					"_crud/{$template}",
					"../third_party/isolde_core/views/_crud/{$template}",
				)
			);
		}
		foreach ($template_candidates as $candidate)
		{
			$file = APPPATH."views/{$candidate}.php";
			if (file_exists($file))
			{
				$selected_template = $candidate;
				break;
			}
			$files[] = $file;
		}
		if ( ! isset($selected_template))
		{
			if ($ignore)
			{
				return;
			}
			show_error(
				'template not found.<br />'.
				'candidates:<br />'.
				'- '.implode('<br />- ', $files).'<br />'
			);
		}

		if ($return === TRUE)
		{
			ob_start();
		}
		require APPPATH."views/{$selected_template}.php";
		if ($return === TRUE)
		{
			$buffer = ob_get_contents();
			ob_end_clean();
			return $buffer;
		}
	}
}

if ( ! function_exists('href'))
{
	function href($uri = '', $return = FALSE)
	{
		$CI =& get_instance();

		$uri = "{$CI->isolde->segments->actor}/{$uri}";
		$uri = preg_replace('/^[^\/]*\/\.\.\//', '', $uri);
		$uri = preg_replace('/^default$/', '', $uri);
		$uri = preg_replace('/^default\//', '', $uri);
		$uri = site_url($uri);
		if ( ! $return)
		{
			echo $uri;
		}
		else
		{
			return $uri;
		}
	}
}

if ( ! function_exists('direct'))
{
	function direct($uri = '', $return = FALSE)
	{
		$uri = href("../_direct/{$uri}");

		return $uri;
	}
}

if ( ! function_exists('form_value'))
{
	function form_value($name, $value = '', $return = FALSE)
	{
		$CI =& get_instance();

		$form_values = unserialize($CI->session->flashdata('form_values'));
		if (isset($form_values->$name))
		{
			$value = $form_values->$name;
		}
		$value = htmlspecialchars($value);

		if ( ! $return)
		{
			echo $value;
		}
		else
		{
			return $value;
		}
	}
}

if ( ! function_exists('condition_value'))
{
	function condition_value($name, $value = '', $return = FALSE)
	{
		$CI =& get_instance();

		$condition_values = unserialize($CI->session->userdata('condition_values'));
		if (isset($condition_values->$name))
		{
			$value = $condition_values->$name;
		}
		$value = htmlspecialchars($value);

		if ( ! $return)
		{
			echo $value;
		}
		else
		{
			return $value;
		}
	}
}


if ( ! function_exists('lang_val'))
{
	function lang_val($line, $values = array(), $return = FALSE)
	{
		if ( ! is_array($values))
		{
			$values = array($values);
		}

		$lang_val = lang($line);
		foreach ($values as $key => $value)
		{
			$lang_val = str_replace('{$'.($key + 1).'}', $value, $lang_val);
		}

		if ( ! $return)
		{
			echo $lang_val;
		}
		else
		{
			return $lang_val;
		}
	}
}

if ( ! function_exists('relaxed_json_decode'))
{
	function relaxed_json_decode($json)
	{
		$json = preg_replace('/#[^\n]*\n/', '', $json);
		$json = preg_replace('/"/', '\"', $json);
		$json = preg_replace('/,\s*([\]}])/', '$1', $json);
		$json = preg_replace('/,\s*$/', '', $json);
		$json = str_replace("'", '"', $json);
		$json = preg_replace('/(\w+):/', '"$1":', $json);
		$value = json_decode('{'.$json.'}', TRUE);

		return $value;
	}
}

if ( ! function_exists('relaxed_json_encode'))
{
	function relaxed_json_encode($value)
	{
		$json = json_encode($value);
		$json = preg_replace('/"(\w+)":/', '$1:', $json);
		$json = str_replace('"', "'", $json);
		$json = preg_replace('/$/', ',', $json);
		$json = preg_replace('/([\]}])/', ',$1', $json);
		$json = preg_replace('/([\[{])/', "$1\n", $json);
		$json = preg_replace('/([\]}])/', "\n$1", $json);
		$json = preg_replace('/([\]}],)/', "$1\n", $json);
		$json = preg_replace('/\n+/', "\n", $json);
		$json = preg_replace('/\n,\n/', "\n", $json);
		$json = preg_replace('/([:,])/', '$1 ', $json);

		$array = explode("\n", $json);
		$t = 0;
		foreach ($array as $key => $value)
		{
			if (preg_match('/^[\]}]/', $value))
			{
				$t--;
			}
			$value = $array[$key] = str_repeat("\t", $t).$value;
			if (preg_match('/[\[{]$/', $value))
			{
				$t++;
			}
		}
		$json = implode("\n", $array);
		while (preg_match('/\t\n/', $json))
		{
			$json = preg_replace('/\t\n/', "\n\t", $json);
		}

		return $json;
	}
}

if ( ! function_exists('mail_utf8'))
{
	function mail_utf8($to_list, $subject, $message, $from = NULL, $from_label = NULL, $cc_list = NULL, $bcc_list = NULL)
	{
		if (is_array($to_list))
		{
			$to = implode(', ', $to_list);
		}
		else
		{
			$to = $to_list;
		}
		$additional_headers = '';
		if (isset($from))
		{
			if (isset($from_label))
			{
				$from = mb_encode_mimeheader(mb_convert_encoding($from_label, 'JIS', 'UTF-8'))."<{$from}>";
			}
			$additional_headers .= "From: {$from}\n";
		}
		if (isset($cc_list))
		{
			if (is_array($cc_list))
			{
				$additional_headers .= "Cc: ".implode(', ', $cc_list)."\n";
			}
			else
			{
				$additional_headers .= "Cc: {$cc_list}\n";
			}
		}
		if (isset($bcc_list))
		{
			if (is_array($bcc_list))
			{
				$additional_headers .= "Bcc: ".implode(', ', $bcc_list)."\n";
			}
			else
			{
				$additional_headers .= "Bcc: {$bcc_list}\n";
			}
		}
		mb_send_mail($to, $subject, $message, $additional_headers);
	}
}
