<?php
///
// Isolde (Revision r168core-r128app)
//
// A third party package for CodeIgniter
//
// Copyright (c) 2013 Kenta Saito (gatedruid.com)
// License       MIT License (See ../isolde_license.txt)
//
?>
<?php
class Isolde_core {

	public $modules = array();

	public $segments;

	public $auth;

	public $crud;

	public $config;

	public function __construct()
	{
		$this->CI =& get_instance();

		$this->CI->config->load('isolde');
		$this->CI->load->database();
		$this->CI->load->library('encrypt');
		$this->CI->load->library('session');
		$this->CI->load->helper('url');
		$this->CI->load->helper('language');
		$this->CI->load->helper('isolde');
		$this->CI->lang->load('isolde');

		$this->segments = (object) array();
		$this->segments->actor =
			in_array($this->CI->uri->segment(1), $this->CI->config->item('actors')) ?
			$this->CI->uri->segment(1) :
			'default'
		;
		$this->segments->target = $this->CI->uri->rsegment(1);
		$this->segments->action = $this->CI->uri->rsegment(2);
		$this->segments->id = $this->CI->uri->rsegment(3);

		if ($this->segments->target == 'crud')
		{
			$this->segments->target = $this->CI->uri->rsegment(4);
		}

		// Auth
		$this->auth = FALSE;
		if (in_array($this->segments->actor, $this->CI->config->item('authentication')))
		{
			$this->enable_auth();
		}

		// Config
		require APPPATH.'config/isolde.php';
		$this->config = (object) $config;
	}

	public function enable_auth()
	{
		$this->CI->load->model('auth_model');

		if ( ! $this->CI->session->userdata('auth'))
		{
			$this->CI->session->set_userdata('auth', (object) array());
		}
		if (isset($this->CI->session->userdata('auth')->{$this->segments->actor}))
		{
			$this->auth = $this->CI->session->userdata('auth')->{$this->segments->actor};
		}
		if ($this->auth && $this->segments->target == 'auth' && $this->segments->action == 'index')
		{
			redirect($this->segments->actor);
		}
		if ( ! $this->auth && $this->segments->target != 'auth')
		{
			redirect("{$this->segments->actor}/auth");
		}
		$this->modules[] = 'auth';
	}

	public function enable_crud()
	{
		$actions = $this->CI->config->item('action_list');
		if (isset($this->CI->config->item('actions')[$this->segments->actor][$this->segments->target]))
		{
			$actions = $this->CI->config->item('actions')[$this->segments->actor][$this->segments->target];
		}
		if ( ! in_array(preg_replace('/^affect_/', '', $this->segments->action), $actions))
		{
			show_404();
		}

		$this->crud = (object) array();
		$this->load_model();
		$this->modules[] = 'crud';
	}

	public function load_model($target = FALSE)
	{
		if ($target === FALSE)
		{
			$target = $this->segments->target;
		}

		if (file_exists(APPPATH."models/{$this->segments->actor}/{$target}_model.php"))
		{
			$this->CI->load->model("{$this->segments->actor}/{$target}_model", 'crud_model');
		}
		else if (file_exists(APPPATH."models/{$target}_model.php"))
		{
			$this->CI->load->model("{$target}_model", 'crud_model');
		}
		else if (file_exists(APPPATH."models/{$this->segments->actor}/crud_model.php"))
		{
			$this->CI->load->model("{$this->segments->actor}/crud_model", 'crud_model');
		}
		else
		{
			$this->CI->load->model('crud_model');
		}
		$this->crud->targets = $this->CI->crud_model->list_targets();
	}

	public function load_view($view, $vars = array(), $return = FALSE)
	{
		$root_template_candidates = array(
			"{$this->segments->actor}/{$this->segments->target}/{$this->segments->action}",
			"{$this->segments->target}/{$this->segments->action}",
			"../third_party/isolde_core/views/{$this->segments->target}/{$this->segments->action}",
		);
		if (in_array('auth', $this->modules) && $this->segments->target == 'auth')
		{
			$root_template_candidates = array_merge(
				$root_template_candidates,
				array(
					"_auth/{$this->segments->action}",
					"../third_party/isolde_core/views/_auth/{$this->segments->action}",
				)
			);
		}
		if (in_array('crud', $this->modules))
		{
			$root_template_candidates = array_merge(
				$root_template_candidates,
				array(
					"{$this->segments->actor}/_crud/{$this->segments->action}",
					"_crud/{$this->segments->action}",
					"../third_party/isolde_core/views/_crud/{$this->segments->action}",
				)
			);
		}
		foreach ($root_template_candidates as $candidate)
		{
			$file = APPPATH."views/{$candidate}.php";
			if (file_exists($file))
			{
				$root_template = $candidate;
				break;
			}
			$files[] = $file;
		}
		if ( ! isset($root_template))
		{
			show_error(
				'root template not found.<br />'.
				'candidates:<br />'.
				'- '.implode('<br />- ', $files).'<br />'
			);
		}

		$isolde = (object) array();

		$isolde->modules = $this->modules;
		$isolde->segments = $this->segments;
		$isolde->auth = $this->auth;
		$isolde->crud = $this->crud;
		$isolde->config = $this->config;

		$vars = (object) array(
			'root_template' => $root_template,
			'isolde' => $isolde,
			'flash_messages' => $this->CI->session->flashdata('messages'),
			'vars' => (object) $vars,
		);

		$view_candidates = array(
			"{$this->segments->actor}/{$this->segments->target}/{$view}",
			"{$this->segments->actor}/{$view}",
			"{$view}",
		);
		if (in_array('auth', $this->modules))
		{
			$view_candidates = array_merge(
				$view_candidates,
				array(
					"_auth/{$view}",
				)
			);
		}
		if (in_array('crud', $this->modules))
		{
			$view_candidates = array_merge(
				$view_candidates,
				array(
					"_crud/{$view}",
				)
			);
		}
		$view_candidates = array_merge(
			$view_candidates,
			array(
				"../third_party/isolde_core/views/{$view}",
			)
		);
		foreach ($view_candidates as $candidate)
		{
			$file = APPPATH."views/{$candidate}.php";
			if (file_exists($file))
			{
				$selected_view = $candidate;
				break;
			}
			$files[] = $file;
		}
		if ( ! isset($selected_view))
		{
			show_error(
				'view not found.<br />'.
				'candidates:<br />'.
				'- '.implode('<br />- ', $files).'<br />'
			);
		}
		$this->CI->load->view("{$selected_view}", $vars, $return);
	}

	public function append_flash_message($message, $class = FALSE)
	{
		if ( ! isset($this->flash_messages))
		{
			$this->flash_messages = array();
		}
		$this->flash_messages[] = (object) array(
			'message' => $message,
			'class' => $class,
		);
		$this->CI->session->set_flashdata('messages', $this->flash_messages);
	}

	public function login($login, $password)
	{
		$row = $this->CI->auth_model->authenticate($login, $password);
		if ($row)
		{
			$auth = $this->CI->session->userdata('auth');
			$auth->{$this->segments->actor} = $row;
			$this->CI->session->set_userdata('auth', $auth);

			$this->append_flash_message(lang('logged_in'), 'info');
			redirect($this->segments->actor);
		}
		else
		{
			$this->append_flash_message(lang('the_login_attempt_failed'), 'danger');
			$this->CI->session->set_flashdata('form_values', serialize((object) array('login' => $login)));
			redirect($this->CI->uri->uri_string());
		}
	}

	public function logout()
	{
		$auth = $this->CI->session->userdata('auth');
		unset($auth->{$this->segments->actor});
		$this->CI->session->set_userdata('auth', $auth);

		$this->append_flash_message(lang('logged_out'), 'info');
		redirect("{$this->segments->actor}/auth");
	}
}
